@@ -3,6 +3,7 @@ from telegram import InlineKeyboardButton
 from telegram import InlineKeyboardMarkup
 from telegram.ext import CallbackQueryHandler
 from telegram.ext import Filters
+from telegram.ext import MessageHandler
 from telegram.ext import Updater, CommandHandler, RegexHandler
 import logging
 import configparser
@@ -66,10 +67,12 @@ def generate_invitation(bot, update):
 @user_language
 def mention_toggle(bot, update):
     message = update.message
-    if message.chat.type == 'private':
+    if not utils.is_allow(message.from_user.id):
+        text = _("You aren't allow to use this")
+    elif message.chat.type == 'private':
         text = _('Use this command in the group where you want to activate/disable the notifications')
     else:
-        text = utils.mention_toggle(message.chat_id, message.from_user.id)
+        text = utils.mention_toggle(bot, update, message.chat_id, message.from_user.id)
     bot.sendMessage(message.chat_id, text, reply_to_message_id=message.message_id)
 
 
@@ -93,6 +96,11 @@ def log_error(bot, update, error):
     logger.warning('Update "%s" caused error "%s"' % (update, error))
 
 
+def filter_assign_alias(msg):
+    return msg.reply_to_message and \
+           msg.reply_to_message.from_user.id == msg.bot.id and \
+           msg.reply_to_message.text_markdown.startswith('🙍‍♂️')
+
 @user_language
 def unknown(bot, update):
     bot.sendMessage(chat_id=update.message.chat_id, text=_("Sorry, I didn't understand that command."))
@@ -118,8 +126,7 @@ def main():
     dp.add_handler(RegexHandler(r'(?i).*\@flandas\b', utils.mention_forwarder))
     dp.add_handler(CallbackQueryHandler(utils.callback_query_handler, pass_chat_data=True))
     dp.add_handler(CommandHandler('users', utils.users_tsdb, Filters.user(user_id=ADMIN_ID), pass_chat_data=True))
-    # dp.add_handler(CommandHandler('pl', utils.get_permision_list))
-
+    dp.add_handler(MessageHandler(filter_assign_alias, utils.assign_user_alias_step2, pass_chat_data=True))
 
     # log all errors
     dp.add_error_handler(log_error)
